library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

-- Kronograma eskuz betetzeko erabili dugun ariketa (bigarrena).
-- Diseinuaren helburua: bi zenbaki osoen arteko biderketa egitea
-- Zenbakiak 2rako osagarrian adierazita daude.
-- Diseinua (ASM+PU) eta kronograma betetzeko eskema egelan daude eskura.
-- Deskribapen hau ASM algoritmoari dagokio.

entity AU_OUT_PU_KU is
port (
clk,reset: in std_logic;
ENABLE: in std_logic;
SAMPLE: in std_logic_vector (15 downto 0);
DACDAT: out std_logic;
BCLK: in std_logic;
DACLRC: in std_logic;
READYOUT: out std_logic
);
end AU_OUT_PU_KU;

architecture arki_out of AU_OUT_PU_KU is

  --erazagupenak: osagaiak eta barne seinaleak
component AU_OUT_KU
port (
clk,reset: in std_logic;
ENABLE, AZKENA, DACLRCSINK, BCLKSINK: in std_logic;
LDSAMPLE, LDBEG, DESEZK, DEKBEG, READYOUT, AUKBIT: out std_logic
);
end component;

  signal LDSAMPLE,LDBEG,AUKBIT : std_logic;
  signal DESEZK,DEKBEG: std_logic;
  signal DACLRCSINK, BCLKSINK, AZKENA: std_logic;
  signal Q_KONT: unsigned (4 downto 0);
  signal Q_SAMPLE: std_logic_vector (15 downto 0);
 
begin -- arkitekturaren hasiera

  ku: AU_OUT_KU
  port map (
    clk => clk,
    reset => reset, ENABLE => ENABLE,
    AZKENA => AZKENA, LDSAMPLE => LDSAMPLE, LDBEG => LDBEG, DESEZK => DESEZK, DEKBEG => DEKBEG, AUKBIT => AUKBIT, READYOUT => READYOUT, BCLKSINK => BCLKSINK, DACLRCSINK => DACLRCSINK
    );

-- Desplazamendu erregistroa
   process(clk)
   begin
    if clk'event and clk='1'  then
        if LDSAMPLE='1'  then Q_SAMPLE<=SAMPLE;
        elsif DESEZK='1' then  
                for I in 1 to 15 loop
                    Q_SAMPLE(I)<=Q_SAMPLE(I-1);
                end loop;
            Q_SAMPLE(0)<=Q_SAMPLE(15);
            end if;
        end if;
	end process;



-- D biegonkorrak
process(clk)
begin
	if CLK'event and CLK='1' then
		BCLKSINK <= BCLK;
	end if;
end process;
 

process(clk)
begin
	if CLK'event and CLK='1' then
		DACLRCSINK<= DACLRC;
	end if;
end process;
                    

    -- emaitza-erregistroaren aurreko multiplexorea
	DACDAT<= Q_SAMPLE(15) when AUKBIT='1' else '0';

    -- Kontagailua
    process(clk)
    begin
      if clk'event and clk='1' then
        if LDBEG='1' then Q_KONT<="10000";
          elsif DEKBEG='1' then Q_KONT<= Q_KONT-1;
        end if;
      end if;
    end process;
   AZKENA<='1' when (Q_KONT="00000") else '0';
    

end arki_out;
