library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

-- Kronograma eskuz betetzeko erabili dugun ariketa (bigarrena).
-- Diseinuaren helburua: bi zenbaki osoen arteko biderketa egitea
-- Zenbakiak 2rako osagarrian adierazita daude.
-- Diseinua (ASM+PU) eta kronograma betetzeko eskema egelan daude eskura.
-- Deskribapen hau ASM algoritmoari dagokio.

entity MAIN_PU_KU is
port (
clk,reset: in std_logic;
ENABLE: out std_logic;
ENABLEIN: out std_logic;
SAMPLE: out std_logic_vector (15 downto 0);
SAMPLEIN: in std_logic_vector (15 downto 0);
PLAY, REC, READYOUT, READYIN, FORWARD, BACKWARD: in std_logic;
MEND: out std_logic
);
end MAIN_PU_KU;

architecture arki_main of MAIN_PU_KU is

  -- ERAZAGUPENAK: OSAGAIAK ETA BARNE SEINALEAK
component MAIN_KU
port (
clk,reset: in std_logic;
AMAITU, READYOUT, READYIN, PLAYSINK, RECSINK, FORWARDSINK, BACKWARDSINK: in std_logic; 
ENABLE, ENABLEIN, LD_K, LD_H, COUNT, INKR, DEKR, MEND, WR, AKT: out std_logic 
);
end component;

component RAMSINC
 port (
   clk: in std_logic;
   WE: in std_logic;
   ADDR: in std_logic_vector (15 downto 0);
   DATIN: in std_logic_vector (15 downto 0);
   DATOUT: out std_logic_vector (15 downto 0)
 );
end component;

  signal AMAITU, PLAYSINK, RECSINK, FORWARDSINK, BACKWARDSINK, WR: std_logic;
  signal LD_K, LD_H, COUNT, INKR, DEKR, GORDE, AKT: std_logic;
  signal HELB, NHELB, MHAS: std_logic_vector (15 downto 0); 


begin -- ARKITEKTURAREN HASIERA
  
  -- MEMORIA
  ram: RAMSINC
  port map (
	clk => clk, WE => WR,
	ADDR => HELB, DATIN => SAMPLEIN, DATOUT => SAMPLE 
   );
   
  ku: MAIN_KU
  port map (
    clk => clk,
    reset => reset, ENABLE => ENABLE, ENABLEIN => ENABLEIN, RECSINK => RECSINK, WR => WR, FORWARDSINK => FORWARDSINK, BACKWARDSINK => BACKWARDSINK,  AKT => AKT, 
    INKR => INKR, DEKR => DEKR, MEND => MEND, AMAITU => AMAITU, LD_K => LD_K, LD_H => LD_H, READYOUT => READYOUT, READYIN => READYIN, PLAYSINK => PLAYSINK, COUNT => COUNT
    );

	 
  -- D BIEGONKORRAK
  process(clk)
  begin
	if CLK'event and CLK='1' then
		PLAYSINK <= PLAY;
		RECSINK <= REC;
		FORWARDSINK <= FORWARD;
		BACKWARDSINK <= BACKWARD;
	end if;
  end process;

    -- KONTAGAILUA
    process(clk)
    begin
      if clk'event and clk='1' then
        if LD_K='1' then HELB<=MHAS;
          elsif COUNT='1' then HELB <= std_logic_vector(unsigned(HELB)+1);
        end if;
      end if;
    end process;
   AMAITU<='1' when (HELB="1111111111111111") else '0';
	
	-- FORWARD/BACKWARD KONTAGAILUA
    process(clk)
    begin
      if clk'event and clk='1' then
        if LD_H='1' then NHELB<=HELB;
          elsif INKR='1' then NHELB <= std_logic_vector(unsigned(HELB)+"0000111111100101");
			 elsif DEKR='1' then NHELB <= std_logic_vector(unsigned(HELB)-"0001111110010100");
        end if;
      end if;
    end process;
   
  
    -- MULTIPLEXOREA   
    MHAS <= NHELB when AKT='1' else "0000000000000000";
   

end arki_main;
